#include <GLUT/glut.h>									//GLUT library
#include <iostream>
#include "Volume.h"
#include "VoxelSurface.h"
#include "VoxelSurfaceRenderer.h"
#include "zpr.h"



Volume<float>*		volume			= 0;
VoxelSurface*		voxelSurface	= 0;
VoxelSurfaceRenderer*  renderer		= 0;
float				max_depth		= 0;
int					draw_mode		= 0;
float				volSize			= 0;



static void mouseclick(int button, int state, int x, int y)
{
	int keys = glutGetModifiers();						//The mouse events are interpreted as follows:	
	if (keys & GLUT_ACTIVE_CTRL)						// 
	button = GLUT_MIDDLE_BUTTON;						// -left button + move:                         rotate viewpoint
	if (keys & GLUT_ACTIVE_SHIFT)						// -middle button (or left button+Control key): zoom in / out
	button = GLUT_RIGHT_BUTTON;							// -right buttom (or left button+Shift key):    translate viewpoint

	zprMouse(button,state,x,y);							//Use the ZPR library to manipulate the viewpoint. The library sets the modelview  
														//OpenGL matrix based on the mouse events, thereby changing the current viewpoint.
}



static void mousemotion(int x, int y)
{
	zprMotion(x,y);
	glutPostRedisplay();
}


static void viewing(int width, int height)					//GLUT window-reshape callback
{
	glutReshapeWindow(width,height);  
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60.0,double(width)/height,10.0f,max_depth);
	glViewport(0,0,width,height);
}


static void keyboard(unsigned char ch,int,int)				//GLUT keyboard callback
{
	switch(ch)
	{
	case ' ':												//Cycle through the voxel-surface's drawing modes
		draw_mode = (++draw_mode) % 4;	
		renderer->draw_mode = (VoxelSurfaceRenderer::DRAW_MODE)draw_mode;
		switch (renderer->draw_mode)
		{
		case VoxelSurfaceRenderer::DRAW_VOXELS: 
		  cout<<"Drawing surface as voxels"<<endl; break;
		case VoxelSurfaceRenderer::DRAW_SPLATS:
		  cout<<"Drawing surface as oriented point-splats"<<endl; break;
		case VoxelSurfaceRenderer::DRAW_GEOM_BALLS:
		  cout<<"Drawing surface as geometric balls"<<endl; break;
		case VoxelSurfaceRenderer::DRAW_SPRITE_BALLS:
		  cout<<"Drawing surface as sprite-rendered balls"<<endl; break;
		}        
		break;
	case 'r':												//Reset the viewpoint
	case 'R':
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		zprInit(0,0,-(1.7f-0.5f)*volSize);
		break;	
	case 'c':
	case 'C':
		renderer->use_colormap = !renderer->use_colormap;
		break;	
	}
	
	glutPostRedisplay();	
}		


static void display()										//GLUT display callback
{
   const int width = volume->getWidth(), height = volume->getHeight(), depth = volume->getDepth();

   glMatrixMode(GL_MODELVIEW);	
   glPushMatrix();
   glTranslatef(-width/2,-height/2,-1.7*depth);				//Translate the volume so it's centered and in front of the near clipping plane
   renderer->draw(*voxelSurface);							//Draw the voxel surface
   glPopMatrix();
}



void createScalarValues(VoxelSurface& surface)				//Create a fancy scalar volume and cut through it
{															//with the given surface
  int width  = surface.getWidth();
  int height = surface.getHeight();
  int depth  = surface.getDepth();
  
  Volume<float> scalars(width,height,depth);				//Create a volume of the same size as the surface's extent
															//(so that cutting works OK)
  for(int i=0;i<depth;i++)					
    for(int j=0;j<height;j++)
      for(int k=1;k<width;k++) 
      {														//Initialize the volume with some scalar function. Try here
		coord3 c(k,j,i);									//your own scalar functions!
		scalars(c) = sin(i/(float(depth))+cos(j/(float)height))+k/float(width);				
	  }
  
  surface.cut(scalars);										//Cur through the volume with our surface.
}


int main(int argc,char* argv[])								//Main program
{
	cout<<"1. Voxel surface visualization and volume-cutting"<<endl;
	cout<<"In the graphics window, press"<<endl;
	cout<<"  -space:   toggle through surface rendering modes"<<endl;
	cout<<"  -c,C:     toggle scalar color-mapping on surface"<<endl;
	cout<<"  -r,R:     reset viewpoint"<<endl;
	
	
	const char* filename = (argc>1)? argv[1]:"DATA/horse.fld";
	
	glutInit(&argc, argv);									//1. Initialize the GLUT toolkit
	glutInitDisplayMode(GLUT_RGB|GLUT_DOUBLE|GLUT_DEPTH);	//2. Ask GLUT to create next windows with a RGB framebuffer and a Z-buffer too
	glutInitWindowSize(500,500);							//3. Tell GLUT how large are the windows we want to create next
	glutCreateWindow("1. Volume visualization");		

	volume = new Volume<float>;

	const int pad = 2;										//4. While reading volumes, pad them with a few zero voxels on all sides
	bool ok;												//   This makes sure that our voxel-surface construction always works OK next
   
   	if(strstr(filename, ".fld"))							//5. Read the volume from an input file
	  ok = volume->readAVSField(filename,pad);
	else if (strstr(filename, ".vtk"))
	  ok = volume->readVTKField(filename,pad);
	else 
	{
	  cout<<"Unknown file format"<<endl;
	  return -1;
	}
	if (!ok)
	{
	  cout<<"Cannot open file: "<<filename<<endl;
	  return -1;
	}
															
	voxelSurface = new VoxelSurface(*volume,0.5);			//6. Extract a voxel-surface by thresholding the volume at some value
	
	createScalarValues(*voxelSurface);						//7. Create some fancy scalar values on the surface by cutting with it through
															//   another scalar volume.
	
	renderer = new VoxelSurfaceRenderer();					//8. Create a renderer for the voxel surface

	volSize = (volume->getWidth()+volume->getHeight()+volume->getDepth())/3.0;

	zprInit(0,0,-(1.7f-0.5f)*volSize);
		
	max_depth = 1.5*sqrt(volume->getWidth()*volume->getWidth() + volume->getHeight()*volume->getHeight() + volume->getDepth()*volume->getDepth());
	   
	glutKeyboardFunc(keyboard);
	glutMouseFunc(mouseclick);
	glutMotionFunc(mousemotion);
	glutReshapeFunc(viewing);
	glutDisplayFunc(display);
	
	glutMainLoop();

	return 0;
}




